﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace DebugExample
{
    class Game
    {
        enum GameState { Menu, CharacterCreator, Game, Battle }
        enum CharacterCreatorStates { EnterName, Display }
        public enum MapPieces { Floor, Wall, Treasure, Enemy }

        string[] mapPiecesString = { "  ", "██", "$$", "><" };
        
        Player player;
        GameState gameState = GameState.Menu;
        CharacterCreatorStates creatorStates = CharacterCreatorStates.EnterName;
        bool gameRunning = true;
        string error = "";
        string battleResult = "";

        int skillPoints = 10;
        int treasure = 0;

        public static Random RNG;
        //Manually created map in code.
        public static int[,] map =
        {
            { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 },
            { 1, 0, 0, 3, 0, 0, 0, 0, 0, 1 },
            { 1, 0, 0, 1, 0, 0, 1, 0, 2, 1 },
            { 1, 0, 0, 1, 3, 1, 1, 1, 1, 1 },
            { 1, 0, 0, 1, 0, 0, 0, 0, 2, 1 },
            { 1, 0, 0, 1, 0, 0, 0, 0, 0, 1 },
            { 1, 0, 0, 1, 1, 1, 3, 1, 1, 1 },
            { 1, 0, 0, 0, 0, 1, 0, 0, 0, 1 },
            { 1, 0, 0, 0, 0, 1, 0, 0, 2, 1 },
            { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 }
        };

        public static Point MapSize;

        public Game()
        {
            MapSize = new Point(map.GetLength(0)-1, map.GetLength(1)-1);
            player = new Player();
            RNG = new Random();
            //When we start we show our "main menu"
            Console.WriteLine("Adventure Quest!\n\nGain treasure and destroy enemies to win!\n\nPress Enter To Begin!");
        }

        public void Run()
        {
            //Game Loop
            while (gameRunning)
            {
                ProcessInput();

                Draw();
            }
        }

        void ProcessInput()
        {
            //Get input from the user
            string currentInput = Console.ReadLine();

            //Process out input differently based on our state
            switch (gameState)
            {
                case GameState.Menu:
                    //If its the menu, we don't care what we type, we're just going to the character creator
                    gameState = GameState.CharacterCreator;
                    break;
                case GameState.CharacterCreator:
                    switch (creatorStates)
                    {
                        case CharacterCreatorStates.EnterName:
                            if(currentInput.Trim(' ') == "")
                            {
                                error = "No Player Name Input";
                            }
                            else
                            {
                                player.name = currentInput;
                                creatorStates = CharacterCreatorStates.Display;
                            }
                            break;
                        case CharacterCreatorStates.Display:
                            //Try catch will catch any errors we have, like if the user typed "Butt", and it couldn't convert that to a number.
                            try
                            {
                                if(currentInput.ToLower() == "r")
                                {
                                    //Go through our points and just assign them randomly
                                    while(skillPoints > 0)
                                    {
                                        player.AddStat((Player.Stats)RNG.Next(0, 4));
                                        skillPoints--;
                                    }
                                    gameState = GameState.Game;
                                }
                                else
                                {
                                    int index = Convert.ToInt32(currentInput) - 1;
                                    //Returns if that index was a valid stat number
                                    if (!player.AddStat((Player.Stats)index))
                                    {
                                        error = "Invalid Stat index, should be 1, 2, 3, or 4";
                                    }
                                    else
                                    {
                                        if (--skillPoints <= 0)
                                        {
                                            gameState = GameState.Game;
                                        }
                                    }
                                }
                            }
                            catch(Exception e)
                            {
                                error = "INVALID INPUT";
                            }
                            break;
                    }
                    break;
                case GameState.Game:
                    MapPieces result = MapPieces.Wall;
                    //Catching the users input to go in the correct directions
                    switch(currentInput.ToLower())
                    {
                        case "W":
                            result = player.Move(Player.Directions.Up);
                            break;
                        case "S":
                            result = player.Move(Player.Directions.Down);
                            break;
                        case "A":
                            result = player.Move(Player.Directions.Left);
                            break;
                        case "D":
                            result = player.Move(Player.Directions.Right);
                            break;
                    }
                    if (result == MapPieces.Treasure)
                    {
                        treasure++;
                        map[player.position.y, player.position.x] = (int)MapPieces.Floor;
                    }
                    else if(result == MapPieces.Enemy)
                    {
                        //Sets up the enemies stats and sends them out to battle
                        uint[] enemyStats = new uint[3];
                        //Enemy gets more to be a challenge
                        int points = 15;
                        while(points > 0)
                        {
                            enemyStats[RNG.Next(0, 4)]++;
                            points--;
                        }
                        battleResult = player.Battle(enemyStats);
                        //If the player didn't die, we get rid of the enemy
                        if (player.health > 0)
                        {
                            map[player.position.y, player.position.x] += (int)MapPieces.Floor;
                        }
                    }
                    break;
            }
        }



        void Draw()
        {
            Console.Clear();
            string displayResult = "Error: No Display";
            switch(gameState)
            {
                case GameState.Menu:
                    break;
                case GameState.CharacterCreator:
                    switch(creatorStates)
                    {
                        case CharacterCreatorStates.EnterName:
                            displayResult = "Enter the name for your character: ";
                            break;
                        case CharacterCreatorStates.Display:
                            displayResult = "Character Creation\n\nSkill Points Available: " +skillPoints + "\n" +
                                "Player: " + player.name + "\n" +
                                "Strength (1): " + player.GetStat(Player.Stats.Strength) + "\n" +
                                "Defense (2): " + player.GetStat(Player.Stats.Defense) + "\n" +
                                "Dexterity (3): " + player.GetStat(Player.Stats.Speed) + "\n" +
                                "Luck (4): " + player.GetStat(Player.Stats.Luck) +
                                "\n\nType stat index to add a skill point. (Or type r for random stats)";
                            break;
                    }
                    break;
                case GameState.Game:
                    //Display stats of the player
                    displayResult = player.name + ": " +
                        " Str(" + player.GetStat(Player.Stats.Strength) + ") " +
                        " Def(" + player.GetStat(Player.Stats.Defense) + ") " +
                        " Dex(" + player.GetStat(Player.Stats.Speed) + ") " +
                        " Luck(" + player.GetStat(Player.Stats.Luck) + ") "+
                        "Treasure: "+treasure+" Health: "+player.health+"/100\n\n";

                    //Key
                    displayResult += "Key: " +
                        player.appearence+" = Player\n" +
                        mapPiecesString[(int)MapPieces.Treasure] + " = Treasure\n"+
                        mapPiecesString[(int)MapPieces.Enemy] + " = Enemy"+
                        "\n\n";

                    //Build map
                    //The map is a little funky, because of the nature of arrays and directions and stuff.
                    //So to get the map to display at same angle as the way I drew it above, I had to draw
                    //it in the reverse x,y of the map. Also had to access it in reverse in other places, which
                    //you may notice.
                    //Another explanatory note, GetLength(0) and GetLength(1) get the sizes of the array. So GetLength(0)
                    //gets the size from the left to right (Like a normal arrays .Length) and GetLength(1) gets it vertically
                    for (int y = 0; y <= map.GetLength(1); y++)
                    {
                        displayResult += "          ";
                        for (int x = 0; x <= map.GetLength(0); x++)
                        {
                            Point current = new Point(x, y);
                            if (current == player.position)
                                displayResult = player.appearence;
                            else
                                displayResult += mapPiecesString[map[y, x]]; 
                        }
                        displayResult += "\n";
                    }

                    //Instructions
                    displayResult += "\n\nWhich direction will you move?\n" +
                        "Up (W)\n" +
                        "Down (S)\n" +
                        "Left (A)\n" +
                        "Right (D)\n";
                    break;
            }

            Console.WriteLine(displayResult + 
                (error != "" ? "\nERROR: "+error : "") + //If we have an error, display it
                (battleResult != "" ? "\n"+battleResult : "")); //If we had a battle result, display it
            battleResult = "";
            error = "";

            //End game displays
            if(player.health < 0)
            {
                Console.WriteLine("\n\nYou've been killed!");
                gameRunning = false;
            }
            if(treasure == 3)
            {
                Console.WriteLine("\n\nYou've gotten all the treasure! Congrats!\nPress Enter To Continue");
                gameRunning = false;
            }
        }

    }
}
